@@ -26,6 +26,7 @@ from tecton.cli import printer
 from tecton.cli.cli_utils import display_principal
 from tecton.cli.cli_utils import display_table
 from tecton.cli.cli_utils import timestamp_to_string
+from tecton.cli.command import TectonCommandCategory
 from tecton.cli.command import TectonGroup
 from tecton.cli.environment_utils import MAX_ENVIRONMENTS_NAME_LENGTH
 from tecton.cli.environment_utils import download_dependencies
@@ -51,6 +52,7 @@ from tecton_proto.remoteenvironmentservice.remote_environment_service__client_pb
 from tecton_proto.remoteenvironmentservice.remote_environment_service__client_pb2 import GetPackagesUploadUrlRequest
 from tecton_proto.remoteenvironmentservice.remote_environment_service__client_pb2 import ListRemoteEnvironmentsRequest
 from tecton_proto.remoteenvironmentservice.remote_environment_service__client_pb2 import StartPackagesUploadRequest
+from tecton_proto.remoteenvironmentservice.remote_environment_service__client_pb2 import UpdateRemoteEnvironmentRequest
 
 
 DEFAULT_DEPENDENCY_RESOLUTION_TOOL = "uv"
@@ -103,7 +105,7 @@ class EnvironmentIdentifier:
         return False
 
 
-@click.command("environment", cls=TectonGroup)
+@click.command("environment", cls=TectonGroup, command_category=TectonCommandCategory.INFRA)
 def environment():
     """Manage Environments for Realtime and Rift job execution.
 
@@ -318,7 +320,7 @@ def _delete(environment_id: Optional[str] = None, name: Optional[str] = None, sk
         confirmation = "y" if skip_confirmation else input(confirmation_text).lower().strip()
         if confirmation == "y":
             try:
-                _delete_environment(environment_id=environment_to_delete.id)
+                delete_environment(environment_id=environment_to_delete.id)
                 printer.safe_print(f"✅ Marked environment '{environment_identifier.__str__()}' for deletion.")
             except Exception as e:
                 printer.safe_print(f"⛔ Failed to delete environment. error = {str(e)}, type= {type(e).__name__}")
@@ -336,6 +338,7 @@ def _display_environments(environments: List, verbose: bool = False):
         "Tecton Runtime Version",
         "Created By",
         "Created At",
+        "Batch Image URI",
     ]
     if verbose:
         headings.extend(["Description", "Status Details"])
@@ -351,6 +354,7 @@ def _display_environments(environments: List, verbose: bool = False):
             i.realtime_job_environment.tecton_transform_runtime_version or "N/A",
             display_principal(i.created_by_principal),
             timestamp_to_string(i.created_at),
+            i.rift_batch_job_environment.image_info.image_uri or "N/A",
         )
         if verbose:
             row += (i.description or "N/A", i.status_details or "N/A")
@@ -411,9 +415,9 @@ def _create_environment_with_requirements(
             printer.safe_print(f"{ERROR_MESSAGE_PREFIX} {e}", file=sys.stderr)
             sys.exit(1)
 
-        assert (
-            tecton_runtime_version or tecton_rift_version
-        ), f"`tecton-runtime` and/or `tecton[rift-materialization]` must be specified in the resolved requirements: \n{resolved_requirements_path.read_text()}"
+        assert tecton_runtime_version or tecton_rift_version, (
+            f"`tecton-runtime` and/or `tecton[rift-materialization]` must be specified in the resolved requirements: \n{resolved_requirements_path.read_text()}"
+        )
 
         printer.safe_print(
             f"\n💡 Creating environment '{name}' for job types:\n"
@@ -467,6 +471,11 @@ def create_remote_environment(**kwargs):
     return metadata_service.instance().CreateRemoteEnvironment(req)
 
 
+def update_environment_status(**kwargs):
+    req = UpdateRemoteEnvironmentRequest(**kwargs)
+    return metadata_service.instance().UpdateRemoteEnvironment(req)
+
+
 def _validate_input_requirements(input_requirements_path: Path):
     reqs = parse_requirements(str(input_requirements_path), session=None)
     for req in reqs:
@@ -477,7 +486,7 @@ def _validate_input_requirements(input_requirements_path: Path):
     printer.safe_print(
         f"{ERROR_MESSAGE_PREFIX} Please include at least one of the supporting Tecton libraries:\n"
         f"💡  `tecton-runtime` package (https://pypi.org/project/tecton-runtime) to support Realtime environments\n"
-        f"💡  `tecton[rift-materialization]` package (https://pypi.org/project/tecton) to support Rift materialization environments\n",
+        f"💡  `tecton\\[rift-materialization]` package (https://pypi.org/project/tecton) to support Rift materialization environments\n",
         "\nFor more information, please see: https://docs.tecton.ai/docs/materializing-features/configure-rift-materialization/python-rift-environments",
         file=sys.stderr,
     )
@@ -584,7 +593,7 @@ def _run_dependency_resolution(
             resolved_requirements_path=resolved_requirements_path,
             timeout_seconds=DEPENDENCY_RESOLUTION_TIMEOUT_SECONDS,
         )
-        dependencies = _validate_and_get_dependencies(lock_output_path)
+        dependencies = _validate_and_get_dependencies(lock_file_path=lock_output_path)
 
         tecton_runtime_version = dependencies.get_version(dependency_name=TECTON_TRANSFORM_RUNTIME_PACKAGE)
         tecton_rift_version = dependencies.get_version(dependency_name=TECTON_RIFT_MATERIALIZATION_RUNTIME_PACKAGE)
@@ -603,7 +612,7 @@ def _run_dependency_resolution(
     return resolved_requirements_path, tecton_runtime_version, tecton_rift_version
 
 
-def _delete_environment(environment_id: str):
+def delete_environment(environment_id: str):
     try:
         req = DeleteRemoteEnvironmentsRequest()
         req.ids.append(environment_id)
@@ -823,7 +832,7 @@ def _validate_and_get_dependencies(lock_file_path: Path) -> EnvironmentDependenc
         rift_extras = dependencies.get_dependency_extras(TECTON_RIFT_MATERIALIZATION_RUNTIME_PACKAGE)
         if "rift-materialization" not in rift_extras:
             printer.safe_print(
-                f"{ERROR_MESSAGE_PREFIX} `rift-materialization` extra must be included for package `tecton` to support Rift materialization (tecton[rift-materialization]==x.x.x)",
+                f"{ERROR_MESSAGE_PREFIX} `rift-materialization` extra must be included for package `tecton` to support Rift materialization (tecton\\[rift-materialization]==x.x.x)",
                 file=sys.stderr,
             )
             sys.exit(1)
@@ -832,7 +841,7 @@ def _validate_and_get_dependencies(lock_file_path: Path) -> EnvironmentDependenc
         printer.safe_print(
             f"{ERROR_MESSAGE_PREFIX} Please include at least one of the supporting Tecton libraries:\n"
             f"💡  `tecton-runtime` package (https://pypi.org/project/tecton-runtime) to support Realtime environments\n"
-            f"💡  `tecton[rift-materialization]` package (https://pypi.org/project/tecton) to support Rift materialization environments",
+            f"💡  `tecton\\[rift-materialization\\]` package (https://pypi.org/project/tecton) to support Rift materialization environments",
             "\nFor more information, please see: https://docs.tecton.ai/docs/materializing-features/configure-rift-materialization/python-rift-environments",
             file=sys.stderr,
         )
