@@ -12,7 +12,6 @@ import numpy
 import pandas
 import pyarrow
 import pyarrow.compute
-from deltalake import DeltaTable
 
 import tecton_core.tecton_pendulum as pendulum
 from tecton_core import conf
@@ -255,6 +254,16 @@ def _agglomerative_clustering_split(
     return split_dfs
 
 
+def validate_spine_input(spine: pyarrow.Table, join_keys: List[str], time_column: str) -> None:
+    missing_join_keys = [key for key in join_keys if key not in spine.column_names]
+    if missing_join_keys:
+        error = f"join_keys must be present in events_df. Missing join keys: {', '.join(missing_join_keys)}"
+        raise ValueError(error)
+    if time_column is not None and time_column not in spine.column_names:
+        error = f"time_column {time_column} must be a column in events_df"
+        raise ValueError(error)
+
+
 def pyarrow_split_spine(
     spine: pyarrow.Table,
     join_keys: List[str],
@@ -327,8 +336,11 @@ def _even_split_pyarrow(
     join_keys: List[str],
     split_count: int,
 ) -> List[pyarrow.Table]:
-    sort_indices = pyarrow.compute.sort_indices(spine, sort_keys=[(key, "ascending") for key in join_keys])
-    spine_sorted = spine.take(sort_indices)
+    # For an RTFV with request source only, no join keys will exist
+    if not join_keys:
+        spine_sorted = spine
+    else:
+        spine_sorted = sort_pyarrow_table_using_duckdb(spine, join_keys)
 
     total_rows = spine_sorted.num_rows
     rows_per_split = total_rows // split_count
@@ -385,6 +397,10 @@ def _calculate_split_count(
 
 # Calculate the total number of rows within the partitions of a Delta table that overlap with a given time range.
 def _get_total_row_number(materialized_data_path: str, spine_time_limits: pendulum.period) -> int:
+    # Keeping this import scoped because deltalake library only installed on python > 3.7
+    # 6.x EMR versions run python 3.7 so will not be able to import this library, but this method only used in Rift
+    from deltalake import DeltaTable
+
     delta_table = DeltaTable(materialized_data_path)
     add_actions = delta_table.get_add_actions(flatten=True).to_pandas()
 
@@ -437,3 +453,26 @@ def merge_validity_periods(df_list, valid_time_range, fdw):
     ).as_ref()
 
     return qt
+
+
+def sort_pyarrow_table_using_duckdb(table: pyarrow.Table, sort_keys: List[str]) -> pyarrow.Table:
+    """
+    Sort a PyArrow table using DuckDB to avoid PyArrow offset overflow issues when sorting datasets with long string values.
+
+    NOTE: DuckDB does not have ns precision with timezones, so we will lose timestamp precision from ns -> us by using DuckDB to convert.
+    This loss in precision is acceptable because we use DuckDB throughout our query tree so the precision inevitably become microseconds.
+    We cast back to nanoseconds in order to keep pyarrow tables consistent for concatenation purposes.
+    """
+    original_schema = table.schema
+    connection = duckdb_factory.create_connection()
+    try:
+        connection.register("input_table", table)
+
+        escaped_keys = ['"' + key.replace('"', '""') + '"' for key in sort_keys]
+        order_by_clause = ", ".join(escaped_keys)
+        query = f"SELECT * FROM input_table ORDER BY {order_by_clause}"
+
+        result = connection.sql(query).arrow()
+        return result.cast(original_schema)
+    finally:
+        connection.close()
